#!/usr/bin/env python   
# -*- coding: utf-8 -*-
import urllib2
from bs4 import BeautifulSoup
import testUrls
import sys
import getRequest

reload(sys)
sys.setdefaultencoding("utf-8")

'''
获取当前页所有链接
'''
def geturls(url):
	source = getsource(url)
	urlset = set()
	try:
		soup = BeautifulSoup(source)
	except:
		return urlset
	for link in soup.find_all('a'):
		if link.get('href') is not None and (link.get('href')[:4] \
			== 'http' or link.get('href')[:1] == '/'):
			if link.get('href')[:4] == 'http':
				urlset.add(link.get('href'))
			if link.get('href')[:1] == '/':
				urlset.add('http://m.sohu.com'+link.get('href'))
	return urlset
'''
获取当前网页源码
'''
def getsource(url):
	
	req, opener = getRequest.getRequest(url)
	try:
		urlopen = opener.open(req, timeout = 10)
		source = urlopen.read()
		return source
	except Exception:
	    testUrls.testUrls(url)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gzip import GzipFile
from StringIO import StringIO
import urllib2

'''
包装urllib2的请求
'''
def getRequest(url):
  encoding_support = ContentEncodingProcessor
  opener = urllib2.build_opener(encoding_support, urllib2.HTTPHandler)
  req = urllib2.Request(url)
  req.add_header('User-Agent','Mozilla/5.0 (Windows;U;Windows NT 5.1;zh-CN;rv:1.9.2.9)Gecko/20100824Firefox/3.6.9')
  return req, opener
  
'''
支持gzip,defalte,加快加载速度
ref:http://www.pythonclub.org/python-network-application/observer-spider
'''

class ContentEncodingProcessor(urllib2.BaseHandler):
  """A handler to add gzip capabilities to urllib2 requests """
 
  # add headers to requests
  def http_request(self, req):
    req.add_header("Accept-Encoding", "gzip, deflate")
    return req

  # decode
  def http_response(self, req, resp):
    old_resp = resp
    # gzip
    if resp.headers.get("content-encoding") == "gzip":
        gz = GzipFile(
                    fileobj=StringIO(resp.read()),
                    mode="r"
                  )
        resp = urllib2.addinfourl(gz, old_resp.headers, old_resp.url, old_resp.code)
        resp.msg = old_resp.msg
    # deflate
    if resp.headers.get("content-encoding") == "deflate":
        gz = StringIO( deflate(resp.read()) )
        resp = urllib2.addinfourl(gz, old_resp.headers, old_resp.url, old_resp.code)  # 'class to add info() and
        resp.msg = old_resp.msg
    return resp

# deflate support
import zlib
def deflate(data):   # zlib only provides the zlib compress format, not the deflate format;
  try:               # so on top of all there's this workaround:
    return zlib.decompress(data, -zlib.MAX_WBITS)
  except zlib.error:
    return zlib.decompress(data)



#!/usr/bin/env python   
# -*- coding: utf-8 -*-

global uv

'''
爬虫线程的链接保存集合，按层分配，一层一个set()
'''
class urlQuence:

    def __init__(self, deepth):
        #已访问的url集合
        self.visited=set()
        #设定爬去深度
        self.deepth = deepth
        #未访问的url字典，按层数做key
        self.unVisited = {}

        for i in range(self.deepth):
            self.unVisited[i+1] = set()

    #获取访问过的url集合
    def getVisitedUrl(self):
        return self.visited
    #获取未访问的url集合
    def getUnvisitedUrl(self):
        uv = set()
        for i, s in self.unVisited.items():
            uv |= s
        return uv
    #添加到访问过得url队列中
    def addVisitedUrl(self, url):
        self.visited.add(url)
    #移除访问过得url
    def removeVisitedUrl(self, url):
        self.visited.remove(url)
    #按层提取未访问过得url
    def unVisitedUrlDeQuence(self, deepth):
        try:
            return self.unVisited[deepth].pop()
        except:
            return None
    #按层插入未访问url
    def addUnvisitedUrl(self, url, deepth):
        self.unVisited[deepth].update(url)
    #获得已访问的url数目
    def getVisitedUrlCount(self):
        return len(self.visited)
    #获得未访问的url数目
    def getUnvistedUrlCount(self):
        count = 0
        for i in range(self.deepth):
            count += len(self.unVisited[i+1])
        return count
    #获取指定层数未访问链接数目
    def getUnvisitedUrlCountByDeepth(self, deepth):
        return len(self.unVisited[deepth])
    #判断未访问的url集合是否为空,返回最小非空层
    def getLayerunVisited(self):
        for i in range(self.deepth):
            if len(self.unVisited[i+1]) != 0:
                return i+1





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import Queue

'''
为检测线程创建一个队列，同时爬虫线程向此队列添加
'''
class urlSync():
	def __init__(self):
		#链接队列
		self.urlQue = Queue.Queue()
		#已检测过的链接
		self.visited = set()
		#临时集合，用来去重
		self.tempSet = set()
	#获取未访问的链接
	def getUnvisitedUrl(self):
		try:
			return self.urlQue.get(block=True, timeout=2.0)
		except Exception, e:
			print "[EXCEPTION] getUnvisitedUrl() - EMPTY"
			return None
	#添加未访问的链接
	def addUnvisitedUrl(self, urls):
		self.tempSet.update(urls)
		for url in (self.tempSet - (self.tempSet & self.visited)):
			self.urlQue.put(url)
		self.tempSet.clear()
	#获取未访问链接数目
	def getUnvistedUrlCount(self):
		return self.urlQue.qsize()
	#获取已访问链接
	def getVisitedUrl(self):
		return self.visited
	#获取访问过的链接数目
	def getVisitedUrlCount(self):
		return len(self.visited)
	#添加已访问过的链接
	def addVisistedUrl(self, url):
		self.visited.add(url)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import urllib2
import threading
import os.path
import time
import getRequest
import socket

global g_mutex
g_mutex  = threading.Lock()

'''
真正干活的检测链接函数，设置超时访问次数为三次
'''
def testUrls(url, repeat = 3):
	ti = time.strftime('%Y-%m-%d %H:%M:%S ',time.localtime(time.time()))
	if url is not None:
		req, opener = getRequest.getRequest(url)
		try:
			urlopen = opener.open(req, timeout = 10)
		except urllib2.HTTPError, ex:
			if(repeat > 0):
				testUrls(url, repeat - 1)
			else:
				log(str(ex), ti, url)
		except urllib2.URLError, ex:
			if(repeat > 0):
				testUrls(url, repeat - 1)
			else:
				log(str(ex), ti, url)
		except socket.timeout as ex:
			if(repeat > 0):
				testUrls(url, repeat - 1)
			else:
				log(str(ex), ti, url)

'''
记录日志
'''
def log(error, ti, url):
	path = getLogsPath()
	print "log-mutex.acquire()..."
	g_mutex.acquire()
	print "log-mutex.acquire() OK"
	fileHandle = open (path+str(time.strftime('%Y-%m-%d'))+'.txt', 'a')
	fileHandle.write(ti+error+url+'\r\n')
	fileHandle.close()
	g_mutex.release()
	print "log-mutex.release() OK"

'''
获取日志路径，若没有则创建
'''
def getLogsPath():
	path = os.path.dirname(__file__)
	newpath = '/'.join(path.split('/')[:-1])+'/logs/'
	if(not os.path.exists(newpath)):
		os.mkdir(newpath)
	return newpath



#!/usr/bin/env python   
# -*- coding: utf-8 -*-
import threading
import sys
sys.path.insert(0, r'..')
import core.bfs as core

'''
爬虫线程
采用广度优先算法
'''
class spliderThread(threading.Thread):
	def __init__(self, urlQuence, urlSync, deepth, mutex):
		threading.Thread.__init__(self)
		self.setDaemon(True)
		#保存爬取链接的集合
		self.urlQuence = urlQuence
		#同时需要更新的供检测线程使用的队列
		self.urlSync = urlSync
		#爬取深度
		self.deepth = deepth
		self.mutex = mutex

	def run(self):
		core.bfs(self.urlQuence, self.urlSync, self.deepth, self.mutex)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
sys.path.insert(0, r'..')
import threading
import utils.testUrls as utils

'''
检测链接的调度器
'''
def test(urlSync, spliderThreadPool, mutex):
    while urlSync.getUnvistedUrlCount() != 0 or spliderIsAlive(spliderThreadPool):
        #print "test-mutex.acquire()..."
        mutex.acquire()
        #print "test-mutex.acquire() OK"
        url = urlSync.getUnvisitedUrl()
        mutex.release()
        #print "test-mutex.release() OK"
        if url in urlSync.visited:
            continue
        if url is None:
            print '剩余链接数量',urlSync.getUnvistedUrlCount()
            for st in spliderThreadPool:
                print st.getName(),st.isAlive()
            continue
        print threading.currentThread()
        #检测链接
        utils.testUrls(url)
        #print "test-2-mutex.acquire()..."
        mutex.acquire()
        #print "test-2-mutex.acquire()OK"
        urlSync.addVisistedUrl(url)
        mutex.release()
        #print "test-2-mutex.release()OK"
        print '剩余链接数量',urlSync.getUnvistedUrlCount()

'''
判断爬虫是否已死
'''
def spliderIsAlive(spliderThreadPool):
    for st in spliderThreadPool:
        if st.isAlive():
            return True
    return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
sys.path.insert(0, r'..')
import utils.getUrls as utils
import threading

'''
广度优先算法，按层提取/存放链接
'''
def bfs(urlQuence, urlSync, deepth, mutex):
    current = urlQuence.getLayerunVisited() #当前未访问的最小层数
    while current is not None and current < deepth+1 :
        #从当前层提取一个未访问链接
        #print "bfs-mutex.acquire()..."
        mutex.acquire()
        #print "bfs-mutex.acquire() OK"
        visitUrl = urlQuence.unVisitedUrlDeQuence(current)
        mutex.release()
        #print "bfs-mutex.release() OK"
        print "第%d层未访问链接出队 \"%s\" "%(current, visitUrl)
        if visitUrl is None or visitUrl=="":
            break
        #获取超链接,set()
        links = utils.geturls(visitUrl)
        #将url放入已访问的url中
        urlQuence.addVisitedUrl(visitUrl)
        #未访问的url入列
        #print "bfs-2-mutex.acquire()..."
        mutex.acquire()
        #print "bfs-2-mutex.acquire() OK"
        urlQuence.addUnvisitedUrl(links, current+1)
        urlSync.addUnvisitedUrl(links)
        mutex.release()
        #print "bfs-2-mutex.release() OK"
        current = urlQuence.getLayerunVisited()



from setuptools import setup, find_packages
import sys, os

version = '0.1'

setup(name='pylinktester',
      version=version,
      description="a link tester written in python",
      long_description="""\
""",
      classifiers=[], # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='python',
      author='yuzhe',
      author_email='lazynight@gmail.com',
      url='http://lazynight.me',
      license='',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          # -*- Extra requirements: -*-
      ],
      entry_points={
      },
      )



#!/usr/bin/env python   
# -*- coding: utf-8 -*-
import spliderThread, testThread
import time
import threading

global g_mutex
g_mutex  = threading.Lock()

'''
线程管理器，初始化爬虫/检测线程，运行完退出
'''
class threadController:
	def __init__(self, threadNumber, urlQuence, urlSync, deepth):
		self.threadNumber = threadNumber
		self.urlQuence = urlQuence
		self.urlSync = urlSync
		self.deepth = deepth
		self.spliderThreadPool = []
		self.testThreadPool = []
		self.mutex = g_mutex
	def start(self):
		for i in range(self.threadNumber):
			self.runSplider()
			if i == 0:	#第一个爬虫需要爬一些预备链接供之后的爬虫使用
				time.sleep(10)
		#等待爬虫线程都开启之后再开启测试线程
		for i in range(self.threadNumber * 2):
			self.runTest()

		self.waitUntilEnd()

	def runSplider(self):
		st = spliderThread.spliderThread(self.urlQuence, self.urlSync, self.deepth, self.mutex)
		self.spliderThreadPool.append(st)
		st.start()

	def runTest(self):
		tt = testThread.testThread(self.urlSync, self.spliderThreadPool, self.mutex)
		self.testThreadPool.append(tt)
		tt.start()

	def waitUntilEnd(self):
		for st in self.spliderThreadPool:
			st.join()
			print "spliderThread[%s] 爬取完毕!" % st.getName()
		for tt in self.testThreadPool:
			tt.join()
			print "testThread[%s] 检测完毕!" % tt.getName()



#!/usr/bin/env python   
# -*- coding: utf-8 -*-
import threading
import sys
sys.path.insert(0, r'..')
import core.test as core

'''
检测线程
'''
class testThread(threading.Thread):
	def __init__(self, urlSync, spliderThreadPool, mutex):
		threading.Thread.__init__(self)
		self.setDaemon(True)
		#链接队列
		self.urlSync = urlSync
		#爬虫线程池，用来检测爬虫是否已死
		self.spliderThreadPool = spliderThreadPool
		self.mutex = mutex

	def run(self):
		core.test(self.urlSync, self.spliderThreadPool, self.mutex)
		